in(node::Node, graph::DAG) = node in graph.nodes
in(edge::Edge, graph::DAG) = edge in graph.edges

function ==(n1::Node, n2::Node, g::DAG)
   if typeof(n1) != typeof(n2)
      return false
   end
   if !(n1 in g) || !(n2 in g)
      return false
   end

   return n1.task == n2.task && children(n1, g) == children(n2, g)
end

# children = prerequisite nodes, nodes that need to execute before the task, edges point into this task
function children(graph::DAG, node::Node)
   if !(node in graph)
      error("Cannot get children of a node that's not in the given graph")
   end

   result::Vector{Edge}
   for edge in graph.edges
      if (edge[2] == node)
         push!(result, edge)
      end
   end

   return result
end

# parents = subsequent nodes, nodes that need this node to execute, edges point from this task
function parents(graph::DAG, node::Node)
   if !(node in graph)
      error("Cannot get parents of a node that's not in the given graph")
   end

   result::Vector{Edge}
   for edge in graph.edges
      if (edge[1] == node)
         push!(result, edge)
      end
   end

   return result
end

is_entry_node(graph::DAG, node::Node) = size(children(graph, node)) == 0
is_exit_node(graph::DAG, node::Node) = size(parents(graph, node)) == 0

function insert_node(graph::DAG, node::Node)
   push!(graph.nodes, node)
end

function insert_edge(graph::DAG, edge::Edge)
   push!(graph.edges, edge)
end
